'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.authorize = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

var _bcryptjs = require('bcryptjs');

var _bcryptjs2 = _interopRequireDefault(_bcryptjs);

var _nodeBits = require('node-bits');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var findUserFromConfig = function findUserFromConfig(database, req, config) {
  return new Promise(function (resolve, reject) {
    var _config$user = config.user,
        model = _config$user.model,
        key = _config$user.key,
        password = _config$user.password;


    var query = {};
    var username = req.body[key];
    if (!username) {
      reject();
      return;
    }
    query[key] = username.trim();

    database.find(model, { where: query }).then(function (users) {
      if (users.length !== 1) {
        reject();
        return;
      }

      var user = users[0];
      var hash = user[password];
      if (!hash) {
        (0, _nodeBits.logError)('User in the database doesnt have a hashed password');
        reject();
        return;
      }

      var check = req.body[password];
      if (!check) {
        (0, _nodeBits.logError)('Password wasnt supplied by the user');
        reject();
        return;
      }

      _bcryptjs2.default.compare(check, hash).then(function (result) {
        if (!result) {
          reject();
          return;
        }

        resolve(user);
      });
    });
  });
};

var mapAttributes = function mapAttributes(object, attributes) {
  return _lodash2.default.reduce(attributes || [], function (result, key) {
    return _extends({}, result, _defineProperty({}, key, object[key]));
  }, {});
};

var runMap = function runMap(object, map) {
  if (!map) {
    return object;
  }

  if (_lodash2.default.isArray(map)) {
    return mapAttributes(object, map);
  }

  if (_lodash2.default.isFunction(map)) {
    return map(object);
  }

  return { map: object[map] };
};

var authorize = exports.authorize = function authorize(config, database) {
  return function (req, res) {
    var secret = config.secret,
        expiresIn = config.expiresIn,
        _config$findUser = config.findUser,
        findUser = _config$findUser === undefined ? findUserFromConfig : _config$findUser;


    findUser(database, req, config).then(function (user) {
      // both, or only one can be specified
      var tokenConfig = config.tokenData || config.returnData;
      var returnConfig = config.returnData || config.tokenData;

      // structure can be an array, or a function
      var tokenData = runMap(user, tokenConfig);
      var returnData = runMap(user, returnConfig);

      // make a token, and return
      var token = _jsonwebtoken2.default.sign(tokenData, secret, { expiresIn: expiresIn });
      res.json({
        success: true,
        token: token,
        returnData: returnData
      });
    }).catch(function (err) {
      (0, _nodeBits.logError)(err);
      res.status(403).json({ success: false });
    });
  };
};